@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: okt 20 2021 (10:48) 
 ## Version: 
-## Last-Updated: jul 31 2023 (18:10) 
+## Last-Updated: maj 10 2024 (16:12) 
 ##           By: Brice Ozenne
-##     Update #: 41
+##     Update #: 62
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -107,5 +107,44 @@ model.tables.mlmm <- function(x, columns, method = NULL, ...){
     return(out)
 }
 
+## * model.tables.effect_lmm
+##' @export
+model.tables.effect_lmm <- function(x, columns, ...){
+
+    extra.var <- c(x$args$variable,unlist(x$args$time),x$args$strata)
+    
+    ## ** usual model.tables
+    newcolumns <- c("estimate","se","df","lower","upper","p.value")
+
+    if(!missing(columns)){
+        
+        if(!is.null(names(columns)) && all(names(columns)=="add")){
+            newcolumns <- union(c(newcolumns, extra.var), unname(columns))
+        }else if(!is.null(names(columns)) && all(names(columns)=="remove")){
+            newcolumns <- setdiff(c(newcolumns, extra.var), unname(columns))
+        }else{
+            newcolumns <- c(newcolumns, extra.var)
+        }
+        if(any(newcolumns %in% extra.var)){
+            add <- x$univariate[intersect(newcolumns,extra.var)]
+            newcolumns <- setdiff(newcolumns,extra.var)
+        }else{
+            add <- NULL
+        }
+    }else{
+        add <- x$univariate[extra.var]
+    }
+
+    out <- cbind(add,
+                 confint(x, ..., columns = newcolumns))
+    rownames(out) <- NULL
+    attr(out, "backtransform") <- NULL
+    attr(out, "error") <- NULL
+    attr(out, "level") <- NULL
+    attr(out, "method") <- NULL
+    class(out) <- "data.frame"
+
+    return(out)
+}
 ##----------------------------------------------------------------------
 ### model.tables.R ends here
