@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: mar  5 2021 (21:39) 
 ## Version: 
-## Last-Updated: jan 10 2024 (15:18) 
+## Last-Updated: May  9 2024 (13:18) 
 ##           By: Brice Ozenne
-##     Update #: 245
+##     Update #: 252
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -77,7 +77,7 @@ print.lmm <- function(x, ...){
             }else if(structure$type == "homogeneous0"){
                 cat("\t\tLinear Mixed Model with ",txt.strata," crossed compound symmetry covariance matrix \n", sep = "")
             }
-        }else if(inherits(structure,"TOEPLITZ")){
+        }else if(inherits(structure,"TOEPLITZ")){            
             if(is.na(structure$name$strata)){
                 txt.strata <- "a"
             }else{
@@ -87,7 +87,7 @@ print.lmm <- function(x, ...){
                 cat("\t\tLinear Mixed Model with ",txt.strata," Toeplitz covariance matrix \n", sep = "")
             }else if(structure$type == "heterogeneous"){
                 cat("\t\tLinear Mixed Model with ",txt.strata," unstructured covariance matrix with constant subdiagonal \n", sep = "")                
-            }else if(structure$type == "lag"){
+            }else if(tolower(structure$type) == "lag"){
                 cat("\t\tLinear Mixed Model with ",txt.strata," block Toeplitz covariance matrix \n", sep = "")
             }else if(structure$type == "homogeneous"){
                 cat("\t\tLinear Mixed Model with ",txt.strata," block compound symmetry covariance matrix with specific subdiagonal \n", sep = "")
@@ -265,6 +265,14 @@ print.LRT_lmm <- function(x, ...){
 }
 
 
+## * print.Wald_lmm
+##' @export
+print.effect_lmm <- function(x, ...){
+    dots <- list(...)
+    dots$print <- c(0,0.5)
+    return(do.call(summary, c(list(object = x, legend = FALSE), dots)))
+}
+
 
 ## * print.mlmm
 ##' @export
