@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: mar  5 2021 (12:59) 
 ## Version: 
-## Last-Updated: jul 26 2023 (11:08) 
+## Last-Updated: Mar 26 2024 (09:51) 
 ##           By: Brice Ozenne
-##     Update #: 592
+##     Update #: 599
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -20,7 +20,7 @@
 ##' @description Extract or compute the first derivative of the log-likelihood of a linear mixed model.
 ##' 
 ##' @param x a \code{lmm} object.
-##' @param data [data.frame] dataset relative to which the score should be computed. Only relevant if differs from the dataset used to fit the model.
+##' @param newdata [data.frame] dataset relative to which the score should be computed. Only relevant if differs from the dataset used to fit the model.
 ##' @param indiv [logical] Should the contribution of each cluster to the score be output? Otherwise output the sum of all clusters of the derivatives.
 ##' @param effects [character] Should the score relative to all coefficients be output (\code{"all"}),
 ##' or only coefficients relative to the mean (\code{"mean"} or \code{"fixed"}),
@@ -42,7 +42,7 @@
 
 ## * score.lmm (code)
 ##' @export
-score.lmm <- function(x, effects = "mean", data = NULL, p = NULL, indiv = FALSE, transform.sigma = NULL, transform.k = NULL, transform.rho = NULL, transform.names = TRUE, ...){
+score.lmm <- function(x, effects = "mean", newdata = NULL, p = NULL, indiv = FALSE, transform.sigma = NULL, transform.k = NULL, transform.rho = NULL, transform.names = TRUE, ...){
 
     ## ** normalize user input
     dots <- list(...)
@@ -58,45 +58,38 @@ score.lmm <- function(x, effects = "mean", data = NULL, p = NULL, indiv = FALSE,
     effects <- match.arg(effects, c("mean","fixed","variance","correlation"), several.ok = TRUE)
     effects[effects== "fixed"] <- "mean"
 
-    init <- .init_transform(transform.sigma = transform.sigma, transform.k = transform.k, transform.rho = transform.rho, 
-                            x.transform.sigma = x$reparametrize$transform.sigma, x.transform.k = x$reparametrize$transform.k, x.transform.rho = x$reparametrize$transform.rho)
+    init <- .init_transform(p = p, transform.sigma = transform.sigma, transform.k = transform.k, transform.rho = transform.rho, 
+                            x.transform.sigma = x$reparametrize$transform.sigma, x.transform.k = x$reparametrize$transform.k, x.transform.rho = x$reparametrize$transform.rho,
+                            table.param = x$design$param)
     transform.sigma <- init$transform.sigma
     transform.k <- init$transform.k
     transform.rho <- init$transform.rho
     test.notransform <- init$test.notransform
+    if(is.null(p)){
+        theta <- x$param
+    }else{
+        theta <- init$p
+    }    
 
     ## ** extract or recompute score
-    if(is.null(data) && is.null(p) && (indiv == FALSE) && test.notransform){
-            keep.name <- stats::setNames(names(coef(x, effects = effects, transform.sigma = "none", transform.k = "none", transform.rho = "none", transform.names = TRUE)),
-                                                     names(coef(x, effects = effects, transform.sigma = transform.sigma, transform.k = transform.k, transform.rho = transform.rho, transform.names = transform.names)))    
+    if(is.null(newdata) && is.null(p) && (indiv == FALSE) && test.notransform){
+        keep.name <- stats::setNames(names(coef(x, effects = effects, transform.sigma = "none", transform.k = "none", transform.rho = "none", transform.names = TRUE)),
+                                     names(coef(x, effects = effects, transform.sigma = transform.sigma, transform.k = transform.k, transform.rho = transform.rho, transform.names = transform.names)))    
 
-            design <- x$design ## useful in case of NA
-            out <- x$score[keep.name]
-            if(transform.names){
-                names(out) <- names(keep.name)
-            }
+        design <- x$design ## useful in case of NA
+        out <- x$score[keep.name]
+        if(transform.names){
+            names(out) <- names(keep.name)
+        }
     }else{
-
         test.precompute <- !is.null(x$design$precompute.XX) && !indiv
 
-        if(!is.null(data)){
-            design <- stats::model.matrix(x, data = data, effects = "all", simplify = FALSE)
+        if(!is.null(newdata)){
+            design <- stats::model.matrix(x, newdata = newdata, effects = "all", simplify = FALSE)
         }else{
             design <- x$design
         }
-
-        if(!is.null(p)){
-            if(any(duplicated(names(p)))){
-                stop("Incorrect argument \'p\': contain duplicated names \"",paste(unique(names(p)[duplicated(names(p))]), collapse = "\" \""),"\".\n")
-            }
-            if(any(names(x$param) %in% names(p) == FALSE)){
-                stop("Incorrect argument \'p\': missing parameter(s) \"",paste(names(x$param)[names(x$param) %in% names(p) == FALSE], collapse = "\" \""),"\".\n")
-            }
-            p <- p[names(x$param)]
-        }else{
-            p <- x$param
-        }
-        out <- .moments.lmm(value = p, design = design, time = x$time, method.fit = x$args$method.fit,
+        out <- .moments.lmm(value = theta, design = design, time = x$time, method.fit = x$args$method.fit,
                             transform.sigma = transform.sigma, transform.k = transform.k, transform.rho = transform.rho,
                             logLik = FALSE, score = TRUE, information = FALSE, vcov = FALSE, df = FALSE, indiv = indiv, effects = effects,
                             trace = FALSE, precompute.moments = test.precompute, transform.names = transform.names)$score
@@ -108,8 +101,8 @@ score.lmm <- function(x, effects = "mean", data = NULL, p = NULL, indiv = FALSE,
         if(!is.numeric(x$cluster$levels)){
             rownames(out) <- x$cluster$levels[match(1:NROW(out),x$cluster$index)]
         } 
-        out <- addNA(out, index.na = x$index.na,
-                     level = "cluster", cluster = x$cluster)        
+        out <- restaureNA(out, index.na = x$index.na,
+                          level = "cluster", cluster = x$cluster)        
         
     }
 
