@@ -3,7 +3,6 @@ library(testthat)
 library(LMMstar)
 
 library(data.table)
-library(emmeans)
 library(ggplot2)
 library(lava)
 library(lme4)
@@ -17,5 +16,4 @@ library(qqtest)
 
 ## * run tests
 ## setwd("~/Documents/GitHub/LMMstar/tests/")
-## setwd("c:/Users/hpl802/Documents/Github/LMMstar/tests/")
 test_check("LMMstar")
